//
//  CurrentInputController.m
//  CurrentInput
//
//  Created by Phidgets on 2015-07-30.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "CapacitiveTouchController.h"

@implementation CapacitiveTouchController

- (void)windowWillClose:(NSNotification*)aNotification{
    if(ch){
        //Safely close example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetCapacitiveTouch_delete(&ch);
        ch = nil;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetCapacitiveTouch_create(&ch);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = PhidgetCapacitiveTouch_setOnTouchHandler(ch, gotTouchData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = PhidgetCapacitiveTouch_setOnTouchEndHandler(ch, gotTouchEndData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}


#pragma mark Event callbacks

static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}

void gotTouchData(PhidgetCapacitiveTouchHandle phid, void *context, double touchValue){
    [(__bridge id)context performSelectorOnMainThread:@selector(onTouchHandler:)
                                           withObject:[NSNumber numberWithDouble:touchValue]
                                        waitUntilDone:NO];
}

void gotTouchEndData(PhidgetCapacitiveTouchHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onTouchEndHandler)
                                           withObject: nil
                                        waitUntilDone:NO];
}

#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    Phidget_DeviceID deviceID;
    uint32 minDataInterval, maxDataInterval, dataInterval;
    double minTouchInputValueChangeTrigger,maxTouchInputValueChangeTrigger, touchInputValueChangeTrigger;
    double minSensitivity, maxSensitivity, sensitivity;
    
    //Get information from channel which will allow us to configure the GUI properly
    Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    PhidgetCapacitiveTouch_getMaxDataInterval(ch, &maxDataInterval);
    PhidgetCapacitiveTouch_getMinDataInterval(ch, &minDataInterval);
    PhidgetCapacitiveTouch_getDataInterval(ch, &dataInterval);
    PhidgetCapacitiveTouch_getMaxTouchValueChangeTrigger(ch, &maxTouchInputValueChangeTrigger);
    PhidgetCapacitiveTouch_getMinTouchValueChangeTrigger(ch, &minTouchInputValueChangeTrigger);
    PhidgetCapacitiveTouch_getTouchValueChangeTrigger(ch, &touchInputValueChangeTrigger);

    if(deviceID == PHIDID_HIN1000 || deviceID == PHIDID_HIN1001){
        [extraSettingsBox setHidden:NO];
        PhidgetCapacitiveTouch_getMinSensitivity(ch, &minSensitivity);
        PhidgetCapacitiveTouch_getMaxSensitivity(ch, &maxSensitivity);
        PhidgetCapacitiveTouch_getSensitivity(ch, &sensitivity);
        
        //Configure sensitivity slider with information read from channel
        [minSensitivityLabel setStringValue:[NSString stringWithFormat:@"%.0f", minSensitivity]];
        [maxSensitivityLabel setStringValue:[NSString stringWithFormat:@"%.0f",maxSensitivity]];
        [sensitivityText setStringValue:[NSString stringWithFormat:@"%.2f",sensitivity]];
        [sensitivitySlider setMinValue:minSensitivity];
        [sensitivitySlider setMaxValue:maxSensitivity];
        [sensitivitySlider setDoubleValue:sensitivity];
    }
    
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [dataBox setHidden:NO];
    [settingsBox setHidden:NO];
    [stateBox setHidden:NO];
    
    
    //Configure data interval slider with information read from channel
    [maxDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",maxDataInterval]];
    [minDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",minDataInterval]];
    [dataIntervalSlider setMaxValue:(double)maxDataInterval];
    [dataIntervalSlider setMinValue:(double)minDataInterval];
    [dataIntervalSlider setIntValue:dataInterval];
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", dataInterval]];
    
    //Configure trigger slider with information read from channel
    [maxChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.3f",maxTouchInputValueChangeTrigger]];
    [minChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.3f",minTouchInputValueChangeTrigger]];
    [changeTriggerSlider setMaxValue:maxTouchInputValueChangeTrigger];
    [changeTriggerSlider setMinValue:minTouchInputValueChangeTrigger];
    [changeTriggerSlider setDoubleValue:touchInputValueChangeTrigger];
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.3f", touchInputValueChangeTrigger]];

    //Adjusting view height
    NSRect frame = [mainWindow frame];
    if([extraSettingsBox isHidden])
        frame.size.height = [mainWindow maxSize].height - extraSettingsBox.frame.size.height;
    else
        frame.size.height = [mainWindow maxSize].height;
        
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Taking items out of view
    [settingsBox setHidden:YES];
    [dataBox setHidden:YES];
    [stateBox setHidden:YES];
    [extraSettingsBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [touchInputValueText setStringValue:@"?"];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onTouchHandler:(NSNumber *)touchValue{
    stateText.stringValue = @"T";
    stateBox.fillColor = [NSColor whiteColor];
    stateText.textColor = [NSColor blackColor];
    [touchInputValueText setStringValue:[NSString stringWithFormat:@"%.3f",[touchValue doubleValue]]];
}

- (void)onTouchEndHandler{
    stateText.stringValue = @"F";
    stateBox.fillColor = [NSColor blackColor];
    stateText.textColor = [NSColor whiteColor];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]
    	attributes:@{NSForegroundColorAttributeName: NSColor.controlTextColor}];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)outputLastError {
	const char *errorString;
	char *errorDetailString = NULL;
    size_t errorDetailStringLen = 0;
    PhidgetReturnCode lastErr;
	
    if (!Phidget_getLastError(&lastErr, &errorString, NULL, &errorDetailStringLen)) {
		errorDetailString = malloc(errorDetailStringLen);
		Phidget_getLastError(&lastErr, &errorString, errorDetailString, &errorDetailStringLen);
    	[self outputError:errorDetailString];
    	free(errorDetailString);
    }
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}


#pragma mark GUI controls
- (IBAction)setChangeTrigger:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetCapacitiveTouch_setTouchValueChangeTrigger(ch, [changeTriggerSlider doubleValue]);
        if(result != EPHIDGET_OK){
            [self outputLastError];
            return;
        }
    }
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.3f", [changeTriggerSlider doubleValue]]];
}

- (IBAction)setDataInterval:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetCapacitiveTouch_setDataInterval(ch, [dataIntervalSlider intValue]);
        if(result != EPHIDGET_OK){
            [self outputLastError];
            return;
        }
    }
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", [dataIntervalSlider intValue]]];
}

- (IBAction)setSensitivity:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetCapacitiveTouch_setSensitivity(ch, [sensitivitySlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [sensitivityText setStringValue:[NSString stringWithFormat:@"%.3f",[sensitivitySlider doubleValue]]];
}


@end
